﻿using System;
using System.Windows.Forms;
using ELTE.BoardGame.Model;
using ELTE.BoardGame.Persistence;
using ELTE.BoardGame.View;

namespace ELTE.BoardGame.Presenter
{
    /// <summary>
    /// Táblajáték prezentáció típusa.
    /// </summary>
    public class BoardGamePresenter
    {
        #region Private fields

        private BoardGameModel model; // játékmodell
        private IBoardGameView view; // (fő) nézet
        private SaveFileDialog saveFileDialog; // dialógusok
        private OpenFileDialog openFileDialog;

        #endregion

        #region Constructors

        /// <summary>
        /// Táblajáték prezentáció példányosítása.
        /// </summary>
        /// <param name="model">A játékmodell.</param>
        public BoardGamePresenter(BoardGameModel model)
        {
            if (model == null)
                throw new ArgumentNullException("model", "Nincs megadva játék");

            this.model = model;

            this.model.GameOver += new EventHandler(Model_GameOver);
            this.model.GameWon += new EventHandler<GameWonEventArgs>(Model_GameWon);
        }

        #endregion

        #region Public properties

        /// <summary>
        /// Nézet lekérdezése, vagy beállítása.
        /// </summary>
        public IBoardGameView View
        {
            get
            {
                return this.view;
            }

            set
            {
                if (this.view == value)
                    return;

                if (this.view != null)
                {
                    // eseményekről történő leirakozás, amennyiben volt egy korábbi nézet
                    this.view.GameStarted -= View_GameStarted;
                    this.view.GameStepped -= View_GameStepped;
                    this.view.LoadGame -= View_LoadGame;
                    this.view.SaveGame -= View_SaveGame;
                    this.view.Model = null; // modell törlése
                }

                this.view = value;
                // eseményekre történő feliratkozás
                this.view.GameStarted += View_GameStarted;
                this.view.GameStepped += View_GameStepped;
                this.view.LoadGame += View_LoadGame;
                this.view.SaveGame += View_SaveGame;
                this.view.Model = this.model; // modell beállítása
            }
        }

        #endregion

        #region Model event handlers

        /// <summary>
        /// Játék megnyerésének eseménykezelése.
        /// </summary>
        private void Model_GameWon(object sender, GameWonEventArgs e)
        {
            switch (e.Player)
            {
                case PlayerData.PlayerO:
                    MessageBox.Show("A kör játékos győzött!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    break;
                case PlayerData.PlayerX:
                    MessageBox.Show("A kereszt játékos győzött!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    break;
            }
            this.model.NewGame();
        }

        /// <summary>
        /// Játék végének eseménykezelése.
        /// </summary>
        private void Model_GameOver(object sender, EventArgs e)
        {
            MessageBox.Show("Döntetlen játék!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            this.model.NewGame();
        }

        #endregion

        #region View event handlers

        /// <summary>
        /// Nézet játékindításának eseménykezelése.
        /// </summary>
        private void View_GameStarted(object sender, EventArgs e)
        {
            this.model.NewGame();
            this.view.UpdateViewState();
        }

        /// <summary>
        /// Nézet játékléptetésének eseménykezelése.
        /// </summary>
        private void View_GameStepped(object sender, BoardEventArgs e)
        {
            try
            {
                this.model.StepGame(e.Column, e.Row);
                this.view.UpdateViewState();
            }
            catch
            {

            }
        }

        /// <summary>
        /// Nézet játékbetöltésének eseménykezelése.
        /// </summary>
        private void View_LoadGame(object sender, EventArgs e)
        {
            if (this.openFileDialog == null)
            {
                this.openFileDialog = new OpenFileDialog();
                this.openFileDialog.Filter = "Játékmentések|*.sav";
                this.openFileDialog.Title = "Tic-Tac-Toe - Játék betöltése";
            }

            if (this.openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    this.model.LoadGame(this.openFileDialog.FileName);
                    this.view.UpdateViewState();
                }
                catch (DataException)
                {
                    MessageBox.Show("Hiba keletkezett a betöltés során.", "Tic-Tac-Toe", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }


        /// <summary>
        /// Nézet játékmentésének eseménykezelése.
        /// </summary>
        private void View_SaveGame(object sender, EventArgs e)
        {
            if (this.saveFileDialog == null)
            {
                this.saveFileDialog = new SaveFileDialog();
                this.saveFileDialog.Filter = "Játékmentések|*.sav";
                this.saveFileDialog.Title = "Tic-Tac-Toe - Játék mentése";
            }

            if (this.saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    this.model.SaveGame(this.saveFileDialog.FileName);
                }
                catch (DataException)
                {
                    MessageBox.Show("Hiba keletkezett a mentés során.", "Tic-Tac-Toe", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        #endregion
    }
}
